from asyncio import sleep

from vkbottle.exceptions import VKError
from vkbottle.framework.blueprint.user import Blueprint

user = Blueprint(
    name='rps_error_blueprint'
)


@user.error_handler.error_handler(6)
async def rps_handler(e: VKError):
    await sleep(1)
    return await e.method_requested(**e.params_requested)



from vkbottle.api import UserApi
from vkbottle.framework.framework.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.logger import logger_decorator
from idm_lp.database import Database, TrustedUser
from idm_lp.utils import get_ids_by_message, edit_message, get_full_name_by_member_id, generate_user_or_groups_list

user = Blueprint(
    name='trusted_blueprint'
)


@user.on.message_handler(
    FromMe(),
    text=[
        '<prefix:service_prefix> +дов [id<user_id:int>|<foo>',
        '<prefix:service_prefix> +дов [club<group_id:int>|<foo>',
        '<prefix:service_prefix> +дов https://vk.com/<domain>',
        '<prefix:service_prefix> +дов',
    ]
)
@logger_decorator
async def add_trusted_member_wrapper(
        message: Message,
        domain: str = None,
        user_id: int = None,
        group_id: int = None,
        **kwargs
):
    db = Database.get_current()
    member_id = user_id if user_id else None
    if not user_id and group_id:
        member_id = -group_id

    member_ids = await get_ids_by_message(message, member_id, domain)
    if not member_ids:
        await edit_message(
            message,
            f'⚠ Необходимо указать пользователей'
        )
        return

    member_id = member_ids[0]
    if member_id == await message.api.user_id:
        await edit_message(
            message,
            f'⚠ Нельзя занести себя в дов-лист!'
        )
        return

    if member_id > 0:
        name = f'Пользователь [id{member_id}|{await get_full_name_by_member_id(message.api, member_id)}]'
    else:
        name = f'Группа [club{abs(member_id)}|{await get_full_name_by_member_id(message.api, member_id)}]'

    if member_id in [
        muted_member.member_id
        for muted_member in db.muted_members
        if muted_member.chat_id == message.peer_id
    ]:
        await edit_message(
            message,
            f'⚠ {name} уже в довлисте'
        )
        return
    db.trusted.append(TrustedUser(user_id=member_id))
    db.save()
    await edit_message(
        message,
        f'✅ {name} добавлен в доверенные'
    )


@user.on.message_handler(
    FromMe(),
    text=[
        '<prefix:service_prefix> -дов [id<user_id:int>|<foo>',
        '<prefix:service_prefix> -дов [club<group_id:int>|<foo>',
        '<prefix:service_prefix> -дов https://vk.com/<domain>',
        '<prefix:service_prefix> -дов',
    ]
)
@logger_decorator
async def remove_trusted_member_wrapper(
        message: Message,
        domain: str = None,
        user_id: int = None,
        group_id: int = None,
        **kwargs
):
    db = Database.get_current()
    member_id = user_id if user_id else None
    if not user_id and group_id:
        member_id = -group_id

    member_ids = await get_ids_by_message(message, member_id, domain)
    if not member_ids:
        await edit_message(
            message,
            f'⚠ Необходимо указать пользователей'
        )
        return

    member_id = member_ids[0]

    if member_id > 0:
        name = f'Пользователь  [id{member_id}|{await get_full_name_by_member_id(message.api, member_id)}]'
    else:
        name = f'Группа [club{abs(member_id)}|{await get_full_name_by_member_id(message.api, member_id)}]'

    if member_id not in [
        trusted_member.user_id
        for trusted_member in db.trusted
    ]:
        await edit_message(
            message,
            f'⚠ {name} не в дов-листе'
        )
        return
    trusted = None
    for ign in db.trusted:
        if ign.user_id == member_id:
            trusted = ign
    db.trusted.remove(trusted)
    db.save()
    await edit_message(
        message,
        f'✅ {name} удален из доверенных'
    )


async def show_trusted_members(
        database: Database,
        api: UserApi
) -> str:
    user_ids = [
        trusted.user_id
        for trusted in database.trusted
        if trusted.user_id > 0
    ]
    group_ids = [
        abs(trusted.user_id)
        for trusted in database.trusted
        if trusted.user_id < 0
    ]

    if not user_ids and not group_ids:
        return "📃 Ваш дов-лист пуст"

    message = "📃 Ваш дов-лист\n"
    return await generate_user_or_groups_list(api, message, user_ids, group_ids)


@user.on.message_handler(
    FromMe(),
    text=[
        '<prefix:service_prefix> довы',
    ]
)
@logger_decorator
async def show_trusted_members_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    await edit_message(
        message,
        await show_trusted_members(
            db,
            message.api
        )
    )



import os

import requests
from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp import const
from idm_lp.const import __version__, __author__
from idm_lp.logger import logger_decorator
from idm_lp.database import Database
from idm_lp.utils import edit_message, SemVer

user = Blueprint(
    name='info_blueprint'
)


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> инфо")
@logger_decorator
async def info_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    version_rest = requests.get(const.VERSION_REST).json()

    last_stable = SemVer(version_rest['version'])
    current = SemVer(const.__version__)

    if current < last_stable:
        update_text = (
            f"\n\nДоступно обновление {version_rest['version']}\n"
            f"{version_rest['description']}\n"
            f"{const.GITHUB_LINK}"
        )
        if 'DYNO' in os.environ:
            update_text += (
                "\n\nЧтобы обновить введите !с обновитьлп"
            )
    elif current > last_stable:
        update_text = (
            "\n\n⚠ Обратите внимание! Вы используете экспериментальную не стабильную версию.\n"
            f"Текущая стабильная: {version_rest['version']}"
        )
    else:
        update_text = ""

    text = f"""
    ❤ IDM LP v{__version__} by {__author__}

    ▶ Ключ рукаптчи: {"&#9989;" if db.ru_captcha_key else "&#10060;"}
    ▶ Удаление уведомлений: {"&#9989;" if db.delete_all_notify else "&#10060;"}
    ▶ Выключение уведомлений: {"&#9989;" if db.disable_notifications else "&#10060;"}

    ▶ В игноре: {len(db.ignored_members)}
    ▶ В глобальном игноре: {len(db.ignored_global_members)}
    ▶ В муте: {len(db.muted_members)}
    ▶ Довов: {len(db.trusted)}
    ▶ Алиасов: {len(db.aliases)}
    ▶ Шаблонов для удаления: {len(db.regex_deleter)}
    ▶ РП-команд: {len(db.role_play_commands)}

    ▶ Выходить из бесед: {"&#9989;" if db.auto_exit_from_chat else "&#10060;"}
    ▶ Удалять диалог: {"&#9989;" if db.auto_exit_from_chat_delete_chat else "&#10060;"}
    ▶ Добавлять пригласившего в ЧС: {"&#9989;" if db.auto_exit_from_chat_add_to_black_list else "&#10060;"}
    
    ▶ Повторялка: {"&#9989;" if db.repeater_active else "&#10060;"}
    ▶ Повторялка | Триггер: {db.repeater_word}

    ▶ Заражение в ответ: {"&#9989;" if db.bio_reply else "&#10060;"}
    
    ▶ NoMeta: {"&#9989;" if db.nometa_enable else "&#10060;"}
    ▶ NoMeta | Количество секунд: {db.nometa_delay}
    ▶ NoMeta | Сообщение: <<{db.nometa_message}>>
    ▶ NoMeta | Вложения: {len(db.nometa_attachments)}
        
    ▶ Префикс ДД: {db.dd_prefix}
    ▶ Сервисные префиксы: {' '.join(db.service_prefixes)}
    ▶ Свои префиксы: {' '.join(db.self_prefixes) if db.self_prefixes else ''}
    ▶ Префиксы дежурного: {' '.join(db.duty_prefixes) if db.duty_prefixes else ''}{update_text}
    """.replace('    ', '')
    await edit_message(
        message,
        text
    )



from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.logger import logger_decorator
from idm_lp.database import Database
from idm_lp.rules import DeleteNotifyRule
from idm_lp.utils import edit_message

user = Blueprint(
    name='delete_notify_blueprint'
)


@user.on.message_handler(DeleteNotifyRule())
@logger_decorator
async def delete_notify_wrapper(message: Message):
    await message.api.messages.delete(message_ids=[message.id])


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -уведы")
@logger_decorator
async def activate_delete_all_notify_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.delete_all_notify = True
    db.save()
    await edit_message(
        message,
        "✅ Удаление уведомлений включено"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +уведы")
@logger_decorator
async def deactivate_delete_all_notify_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.delete_all_notify = False
    db.save()
    await edit_message(
        message,
        "✅ Удаление уведомлений отключено"
    )



from vkbottle.exceptions import VKError
from vkbottle.framework.blueprint.user import Blueprint

user = Blueprint(
    name='access_token_error_blueprint'
)


# noinspection PyUnusedLocal
@user.error_handler.error_handler(5)
async def rps_handler(e: VKError):
    exit(1)



class DatabaseWarning(Exception):

    def __init__(self, name: str, description: str):
        self.name = name
        self.description = description


class DatabaseError(Exception):

    def __init__(self, name: str, description: str) -> None:
        self.name = name
        self.description = description



from python_rucaptcha import ImageCaptcha
from vkbottle.exceptions import VKError
from vkbottle.framework.blueprint.user import Blueprint

from idm_lp.database import Database

user = Blueprint(
    name='captcha_error_blueprint'
)


@user.error_handler.captcha_handler
async def solve_captcha(e: VKError):
    db = Database.get_current()

    if not db.ru_captcha_key:
        return

    user_answer = ImageCaptcha. \
        ImageCaptcha(rucaptcha_key=db.ru_captcha_key). \
        captcha_handler(captcha_link=e.raw_error['captcha_img'])

    if not user_answer['error']:
        return user_answer['captchaSolve']



from enum import Enum

from pydantic import BaseModel


class GenEnum(Enum):
    NOM = "nom"
    GEN = "gen"
    DAT = "dat"
    ACC = "acc"
    INS = "ins"
    ABL = "abl"


class RolePlayCommand(BaseModel):
    name: str
    gen: GenEnum
    formatter_man: str
    formatter_woman: str
    all_ending: str



import asyncio
import string
import time

from vkbottle import VKError
from vkbottle.framework.framework.rule import PrivateMessage
from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.database import Database
from idm_lp.logger import logger_decorator
from idm_lp.utils import edit_message

user = Blueprint(
    name='nometa_blueprint'
)


def concat(element_list: list, postfix: str) -> list:
    for item in element_list:
        yield item + postfix


HELLO_TEXTS = [
    "привет", "ку", "салам",
    "приветствую", "здрасте", "здравствуйте",
    "здарова", "йоу", 'хай',
    'добрыйдень', 'доброеутро', 'добрыйвечер',
    'доброговременисуток', 'шалам', 'cаломалейкум',
    'cаламалейкум', 'салам', 'салом', "приветствуювас", "приветствую",
    "мыразделяемстобой солнце", "мыразделяемстобойлуну",
    'бонжур', 'привiт', 'hi', 'hello', 'здравствуй', 'приветик', 'хаюшки'
]

REPLACE_SYMBOLS = string.punctuation + " "

HELLO_STICKER_IDS = [

]


def levenshtein_distance(a: str, b: str) -> int:
    n, m = len(a), len(b)
    if n > m:
        a, b = b, a
        n, m = m, n
    current_row = range(n + 1)
    for i in range(1, m + 1):
        previous_row, current_row = current_row, [i] + [0] * n
        for j in range(1, n + 1):
            add, delete, change = previous_row[j] + 1, current_row[j - 1] + 1, previous_row[j - 1]
            if a[j - 1] != b[i - 1]:
                change += 1
            current_row[j] = min(add, delete, change)

    return current_row[n]


async def nometa_checker(db: Database, message: Message):
    await asyncio.sleep(db.nometa_delay)
    try:
        msgs = await message.api.messages.get_by_conversation_message_id(
            peer_id=message.peer_id,
            conversation_message_ids=[message.conversation_message_id + 1]
        )
        if msgs.items:
            return
    except VKError:
        pass

    await message.api.messages.send(
        peer_id=message.peer_id,
        message=db.nometa_message,
        attachment=",".join(db.nometa_attachments) if db.nometa_attachments else None,
        random_id=0
    )


@user.on.message_handler(FromMe(False), PrivateMessage())
@logger_decorator
async def nometa_message_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    if not db.nometa_enable:
        return

    text_to_scan = message.text.lower()
    for symb in REPLACE_SYMBOLS:
        text_to_scan = text_to_scan.replace(symb, '')

    if min(map(lambda x: levenshtein_distance(text_to_scan, x), HELLO_TEXTS)) > 3:
        return
    try:
        msgs = await message.api.messages.get_by_conversation_message_id(
            peer_id=message.peer_id,
            conversation_message_ids=[message.conversation_message_id - 1]
        )
        if msgs.items:
            time_prev = msgs.items[0].date
            if time.time() - time_prev < 60 * 60 * 12:
                return
    except (VKError, IndexError):
        pass

    asyncio.ensure_future(nometa_checker(db, message))


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -nometa")
@logger_decorator
async def activate_nometa_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.nometa_enable = False
    db.save()
    await edit_message(
        message,
        "✅ Режим NoMeta отключен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +nometa")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.nometa_enable = True
    db.save()
    await edit_message(
        message,
        "✅ Режим NoMeta включен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> nometa сообщение <text>")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, text: str, **kwargs):
    db = Database.get_current()
    db.nometa_message = text
    db.save()
    await edit_message(
        message,
        "✅ Текст NoMeta обновлен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> nometa задержка <delay:float>")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, delay: float, **kwargs):
    db = Database.get_current()
    db.nometa_delay = delay
    db.save()
    await edit_message(
        message,
        "✅ Задержка NoMeta обновлена"
    )



from pydantic import BaseModel


class Alias(BaseModel):
    name: str
    command_from: str
    command_to: str

    @property
    def regexp(self) -> str:
        _regexp_control = r"[]\/^$.|?*+-(){}"
        command_from = self.command_from[:]
        for control in _regexp_control:
            command_from = command_from.replace(control, "\\" + control)
        return command_from



import enum
import typing
import uuid
from datetime import datetime

import pydantic

from idm_lp.utils import parse_cron_text


class TimerType(enum.Enum):
    """
    AT_DATE - выполняется один раз, в какое-то время например 24.06.2023 в 20:20:21
    INTERVAL - выполняется каждые N секунд
    EVERY_DATE - выполняется каждый месяц, 4 числа в 20:20:21
    """
    DATE = "date"
    INTERVAL = "interval"
    CRON = "cron"


class TimerMethod(enum.Enum):
    """
    SEND_MESSAGE - просто отправить смс
    """
    SEND_MESSAGE = "send_message"


class Timer(pydantic.BaseModel):
    id: uuid.UUID = pydantic.Field(default_factory=lambda: uuid.uuid4())
    name: str
    type: TimerType
    method: TimerMethod

    peer_id: typing.Optional[int]
    message_text: typing.Optional[str]
    message_attachment: typing.Optional[str]

    cron: typing.Optional[str]
    run_date: typing.Optional[datetime]
    interval: typing.Optional[int]

    @property
    def cron_params(self):
        """Gentool on https://ru.rakko.tools/tools/88/
        """
        return parse_cron_text(self.cron)

    @property
    def scheduler_params(self):
        data = {
            "id": self.id.hex,
            "name": "Таймер " + self.name,
            "trigger": self.type.value,
        }
        if self.type == TimerType.DATE:
            data['run_date'] = self.run_date
        elif self.type == TimerType.INTERVAL:
            data['seconds'] = self.interval
        else:
            data.update(**self.cron_params)
        return data



import typing
from typing import Optional, List, Iterable

from vkbottle import VKError
from vkbottle.api import UserApi
from vkbottle.user import Message

__all__ = (
    'edit_message',
    'get_id_by_domain',
    'get_ids_by_message',
    'get_full_name_by_member_id',
    'parse_cron_text',
    'generate_user_or_groups_list',
    'SemVer'
)


async def edit_message(
        message: Message,
        text: str = '',
        **kwargs
) -> int:
    """Простой редач сообщений
    Какие параметры пихать -- https://vk.com/dev/messages.edit

    :return: После успешного выполнения возвращает 1.
    """
    kwargs.setdefault('message_id', message.id)
    kwargs.setdefault('message', text)
    kwargs.setdefault('peer_id', message.peer_id)
    kwargs.setdefault('keep_forward_messages', True)
    kwargs.setdefault('keep_snippets', True)
    kwargs.setdefault('dont_parse_links', False)

    return await message.api.messages.edit(
        **kwargs
    )


async def get_id_by_domain(
        api: UserApi,
        domain: str,
        obj_types: Iterable[str] = ('user',)
) -> Optional[int]:
    """Определяет идентификатор обьекта по короткому имени

    :param api: АПИИИИИИИИИИИИИИИ
    :param domain: короткое имя
    :param obj_types: ID каких обьектов возвращать user, group, application
    """
    try:
        search = await api.utils.resolve_screen_name(screen_name=domain)
        if search.type in obj_types:
            return search.object_id if search.type == 'user' else -search.object_id
    except VKError:
        return None


async def get_ids_by_message(
        message: Message,
        member_id: Optional[int] = None,
        domain: Optional[str] = None
) -> List[int]:
    results = []

    if member_id:
        results.append(member_id)

    if domain:
        result = await get_id_by_domain(message.api, domain, ['user', 'group'])
        if result:
            results.append(result)

    if message.reply_message:
        results.append(message.reply_message.from_id)

    if message.fwd_messages:
        for fwd_msg in message.fwd_messages:
            results.append(fwd_msg.from_id)

    return results


async def get_full_name_by_member_id(
        api: UserApi,
        member_id: int
) -> str:
    """Полное имя по ID"""
    if member_id > 0:
        user = (await api.users.get(user_ids=member_id))[0]
        return f"{user.first_name} {user.last_name}"
    else:
        group = (await api.groups.get_by_id(group_ids=abs(member_id)))[0]
        return group.name


def parse_cron_text(cron: str) -> dict:
    """Парс CRON выражения для планировщика задач"""
    second, minute, hour, day, month, day_of_week, year = cron.split(' ')
    return {
        key: value
        for key, value in dict(
            second=second, minute=minute,
            hour=hour, day=day,
            month=month, day_of_week=day_of_week,
            year=year
        ).items()
        if value != "?"
    }


async def generate_user_or_groups_list(
        api: UserApi,
        message: str,
        user_ids: typing.Optional[typing.List[int]] = None,
        group_ids: typing.Optional[typing.List[int]] = None
):
    if user_ids:
        for index, vk_user in enumerate(await api.users.get(user_ids=user_ids), 1):
            message += f"{index}. [id{vk_user.id}|{vk_user.first_name} {vk_user.last_name}]\n"
    if group_ids:
        for index, vk_group in enumerate(await api.groups.get_by_id(group_ids=group_ids), 1):
            message += f'{index}. [club{vk_group.id}|{vk_group.name}]'
            index += 1
    return message


class SemVer:

    def __init__(self, sem_ver: str):
        self.major, self.minor, self.patch = map(lambda x: int(x), sem_ver.split('.'))

    def __eq__(self, other: "SemVer") -> bool:
        return self.major == other.major and self.minor == other.minor and self.patch == other.patch

    def __lt__(self, other: "SemVer") -> bool:
        if self.major < other.major:
            return True
        if self.major == other.major and self.minor < other.minor:
            return True
        return self.major == other.major and self.minor == other.minor and self.patch < other.patch

    def __le__(self, other: "SemVer") -> bool:
        return self.__lt__(other) or self.__eq__(other)

    def __gt__(self, other: "SemVer") -> bool:
        return not self.__lt__(other) and not self.__eq__(other)

    def __ge__(self, other: "SemVer") -> bool:
        return not self.__lt__(other) or self.__eq__(other)



import re
from typing import List, Union

from vkbottle.rule import AbstractMessageRule, Message

from idm_lp.database import Database


class DeleteNotifyRule(AbstractMessageRule):
    notify_all_words = [
        'all',
        'online',
        'here',
        'everyone',

        'все',
        'онлайн',
        'здесь',
        'тут',
    ]

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        if not db.delete_all_notify:
            return False

        if any([f"@{i}" in message.text.lower() for i in self.notify_all_words]):
            return True
        return False


class ChatEnterRule(AbstractMessageRule):
    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for chat_enter_model in db.add_to_friends_on_chat_enter:
            if chat_enter_model.peer_id == message.peer_id:
                return True
        return False


class IgnoredMembersRule(AbstractMessageRule):

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for ignore_member in db.ignored_members:
            if ignore_member.chat_id == message.peer_id and ignore_member.member_id == message.from_id:
                return True
        return False


class IgnoredGlobalMembersRule(AbstractMessageRule):

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for ignore_member in db.ignored_global_members:
            if ignore_member.member_id == message.from_id:
                return True
        return False


class MutedMembersRule(AbstractMessageRule):

    async def check(self, message: Message):
        db = Database.get_current()
        for muted_member in db.muted_members:
            if muted_member.chat_id == message.peer_id and muted_member.member_id == message.from_id:
                return dict(member=muted_member)
        return False


class SlouMoRule(AbstractMessageRule):

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for slou in db.sloumo:
            if slou.chat_id == message.chat_id:
                return True
        return False


class TrustedRule(AbstractMessageRule):

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for trusted in db.trusted:
            if trusted.user_id == message.from_id:
                return True
        return False


class RegexDeleter(AbstractMessageRule):

    async def check(self, message: Message) -> bool:
        db = Database.get_current()
        for regex_del in db.regex_deleter:
            if regex_del.chat_id == message.peer_id:
                if re.findall(regex_del.regex, message.text):
                    return True
        return False


class ContainsRule(AbstractMessageRule):
    not_include: List[str]

    def __init__(self, words: Union[str, List[str]], not_include: List[str] = None, upper: bool = True):
        if not_include is None:
            not_include = []
        self.words = words if isinstance(words, list) else [words]
        self.not_include = not_include if isinstance(not_include, list) else [not_include]
        self.upper = upper

    async def check(self, message: Message) -> bool:
        checked = False
        for word in self.words:
            if word.upper() in message.text.upper():
                checked = True
        for ni_word in self.not_include:
            if ni_word.upper() in message.text.upper():
                checked = False
        return checked



from typing import Optional

from vkbottle import VKError
from vkbottle.rule import ChatActionRule, FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.logger import logger_decorator
from idm_lp.database import Database, ChatEnterModel
from idm_lp.rules import ChatEnterRule
from idm_lp.utils import edit_message

user = Blueprint(
    name='add_to_friends_on_chat_enter'
)


@user.on.chat_message(ChatActionRule(["chat_invite_user", "chat_invite_user_by_link"]), ChatEnterRule())
@logger_decorator
async def chat_enter_wrapper(message: Message):
    db = Database.get_current()
    model = None
    for chat_enter_model in db.add_to_friends_on_chat_enter:
        if chat_enter_model.peer_id == message.peer_id:
            model = chat_enter_model
    try:
        await user.api.friends.add(user_id=message.action.member_id)
    except VKError:
        pass
    return model.hello_text


@user.on.chat_message(FromMe(), text=[
    "<prefix:service_prefix> +добавление",
    "<prefix:service_prefix> +добавление <hello_text>"
])
@logger_decorator
async def add_chat_enter_model_wrapper(message: Message, hello_text: Optional[str] = None, **kwargs):
    db = Database.get_current()
    for i in range(len(db.add_to_friends_on_chat_enter)):
        if db.add_to_friends_on_chat_enter[i].peer_id == message.peer_id:
            db.add_to_friends_on_chat_enter[i].hello_text = hello_text
            db.save()
            await edit_message(
                message,
                "✅ Приветственный текст обновлен"
            )
            return
    db.add_to_friends_on_chat_enter.append(
        ChatEnterModel(peer_id=message.peer_id, hello_text=hello_text)
    )
    db.save()
    await edit_message(
        message,
        "✅ Добавление новичков в друзья в этом чате включено"
    )
    return


@user.on.chat_message(FromMe(), text=[
    "<prefix:service_prefix> -добавление",
])
@logger_decorator
async def add_chat_enter_model_wrapper(message: Message, **kwargs):
    with Database.get_current() as db:
        model = None
        for i in range(len(db.add_to_friends_on_chat_enter)):
            if db.add_to_friends_on_chat_enter[i].peer_id == message.peer_id:
                model = db.add_to_friends_on_chat_enter[i]
        if model is None:
            await edit_message(
                message,
                "⚠ Добавление новичков в друзья в этом чате не настроено"
            )
            return
        db.add_to_friends_on_chat_enter.remove(model)
    await edit_message(
        message,
        "✅ Добавление новичков в друзья в этом чате выключено"
    )
    return



from idm_lp.commands.members_manager import ignored
from idm_lp.commands.members_manager import ignored_global
from idm_lp.commands.members_manager import muted
from idm_lp.commands.members_manager import trusted

users_bp = (
    ignored.user,
    ignored_global.user,
    muted.user,
    trusted.user,
)



import asyncio
import json
import os
import typing
from typing import List

from pydantic import BaseModel, validator, Field

from idm_lp import const
from . import (
    Alias,
    ChatEnterModel,
    IgnoredMembers,
    MutedMembers,
    ContextInstanceMixin,
    RegexDeleter,
    RolePlayCommand,
    TrustedUser,
    SlouMo,
    DatabaseError,
    Timer
)


class Database(BaseModel, ContextInstanceMixin):
    # Не передаются на сервер, получаются либо с него (исключая токены и сервисные префиксы), либо с файла
    tokens: List[str] = Field([], to_server='exclude', from_server='exclude')
    secret_code: str = Field("", to_server='exclude', from_server='include')
    ru_captcha_key: typing.Optional[str] = Field("", to_server='exclude', from_server='include')
    service_prefixes: List[str] = Field([".слп", "!слп"], to_server='exclude', from_server='exclude')

    # Получаются исключительно с сервера
    repeater_word: str = Field("..", to_server='include', from_server='include')
    dd_prefix: str = Field("дд", to_server='include', from_server='include')

    timers: typing.List[Timer] = Field([], to_server='include', from_server='include')

    auto_infection: bool = Field(False, to_server='include', from_server='include')
    auto_infection_interval: int = Field(3600, to_server='include', from_server='include')
    auto_infection_peer_id: int = Field(-174105461, to_server='include', from_server='include')
    auto_infection_argument: str = Field("р", to_server='include', from_server='include')

    bio_reply: bool = Field(False, to_server='include', from_server='include')
    repeater_active: bool = Field(False, to_server='include', from_server='include')

    delete_all_notify: bool = Field(False, to_server='include', from_server='include')
    auto_exit_from_chat: bool = Field(False, to_server='include', from_server='include')
    auto_exit_from_chat_delete_chat: bool = Field(False, to_server='include', from_server='include')
    auto_exit_from_chat_add_to_black_list: bool = Field(False, to_server='include', from_server='include')
    disable_notifications: bool = Field(False, to_server='include', from_server='include')

    nometa_enable: bool = Field(False, to_server='include', from_server='include')
    nometa_message: str = Field("nometa.xyz", to_server='include', from_server='include')
    nometa_attachments: List[str] = Field([], to_server='include', from_server='include')
    nometa_delay: float = Field(5 * 60, to_server='include', from_server='include')

    self_prefixes: List[str] = Field([".л", "!л"], to_server='include', from_server='include')
    duty_prefixes: List[str] = Field([".лд", "!лд"], to_server='include', from_server='include')

    ignored_members: List[IgnoredMembers] = Field([], to_server='include', from_server='include')
    muted_members: List[MutedMembers] = Field([], to_server='include', from_server='include')
    aliases: List[Alias] = Field([], to_server='include', from_server='include')
    role_play_commands: List[RolePlayCommand] = Field([], to_server='include', from_server='include')
    trusted: List[TrustedUser] = Field([], to_server='include', from_server='include')
    add_to_friends_on_chat_enter: List[ChatEnterModel] = Field([], to_server='include', from_server='include')
    sloumo: List[SlouMo] = Field([], to_server='include', from_server='include')
    regex_deleter: List[RegexDeleter] = Field([], to_server='include', from_server='include')

    spy_check_online: List[int] = Field([], to_server='include', from_server='include')
    spy_check_typing: List[int] = Field([], to_server='include', from_server='include')
    spy_check_messages: List[int] = Field([], to_server='include', from_server='include')

    __on_save_listeners: typing.List[typing.Callable] = []

    @property
    def ignored_global_members(self):
        return [x for x in self.ignored_members if x.chat_id is None]

    def __enter__(self) -> "Database":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.save()

    @validator('tokens')
    def name_must_contain_space(cls, v):
        if not v:
            raise DatabaseError(
                name='Нет токенов',
                description='Укажите токены в файле конфигурации'
            )
        return v

    @staticmethod
    def get_path() -> str:
        if const.USE_APP_DATA:
            local_data_path = os.environ["APPDATA"]
            return os.path.abspath(
                os.path.join(
                    local_data_path,
                    "IDM",
                    const.CONFIG_PATH
                )
            )
        return os.path.abspath(const.CONFIG_PATH)

    @staticmethod
    def load() -> 'Database':
        path_to_file = Database.get_path()
        try:
            db = Database.parse_file(path_to_file)
        except FileNotFoundError:
            db = None

        if not db:
            raise DatabaseError(
                'Конфиг не найден',
                f"Конфиг не найден по пути {path_to_file}"
            )

        if not db.tokens:
            raise DatabaseError(
                'Нет токенов',
                f"Укажите токены в файле конфигурации по пути {path_to_file}"
            )
        return db

    @classmethod
    def add_on_save(cls, func):
        cls.__on_save_listeners.append(func)
        return func

    def load_from_server(self):
        from ..idm_api import IDMAPI
        new_config = IDMAPI.get_current().get_lp_info_sync(self.tokens[0])['config']
        new_database = {
            "tokens": self.tokens,
            "service_prefixes": self.service_prefixes,
            "secret_code": self.secret_code,
            **new_config
        }
        return Database.parse_obj(new_database)

    def get_to_server(self):
        to_server = {}
        for key, value in json.loads(self.json()).items():
            try:
                field = self.__fields__[key]
                extra = field.field_info.extra
                if extra['to_server'] == 'exclude':
                    continue
                to_server[key] = value
            except KeyError:
                pass
        return to_server

    def save(self):
        path_to_file = Database.get_path()
        for __on_save_listener in self.__on_save_listeners:
            asyncio.create_task(__on_save_listener(self))

        with open(path_to_file, 'w', encoding='utf-8') as file:
            file.write(
                self.json(exclude={'__on_save_listeners'}, **{"ensure_ascii": False, "indent": 2})
            )



from typing import Dict, Any, List

from pydantic import BaseModel


class LastMessage(BaseModel):
    date: float
    from_id: int

    def save(self) -> Dict[str, Any]:
        return {
            'date': self.date,
            'from_id': self.from_id
        }


class SlouMo(BaseModel):
    chat_id: int
    last_message: LastMessage
    white_list: List[int]
    warn_message: str
    time: int




import datetime
import typing

from vkbottle.framework.framework.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp import const
from idm_lp.database import Database, Timer, TimerMethod, TimerType
from idm_lp.logger import logger_decorator
from idm_lp.utils import edit_message, parse_cron_text

user = Blueprint(
    name='timers_blueprint'
)


class E(Exception):
    def __init__(self, m: str): self.m = m

    def __str__(self): return self.m


def get_arguments(m: str, mode: TimerType) -> typing.Tuple[
    typing.Any,
    typing.Optional[str]
]:
    data = m.split('\n')
    if len(data) < 2:
        raise E("⚠ Необходимо указать дату и сообщение")
    try:
        data, text = data[0], '\n'.join(data[1:])
    except IndexError:
        data = data[0]
        text = None
    if mode == TimerType.INTERVAL:
        try:
            return int(data), text
        except ValueError:
            raise E("⚠ Интервал должен быть числом")
    elif mode == TimerType.CRON:
        try:
            parse_cron_text(data)
            return data, text
        except ValueError:
            raise E("⚠ Не верное CRON выражение.")
    else:
        try:
            return datetime.datetime.strptime(data, '%Y.%m.%d %H:%M:%S'), text
        except ValueError:
            raise E("⚠ Дата должна быть в формате 2022.03.16 12:46:22")


def check_name(db: Database, name: str):
    if name.lower() in db.timers:
        raise E("⚠ Такой таймер уже существует")


async def create_timer(message: Message, name, text_message, timer_type: TimerType):
    with Database.get_current() as db:
        try:
            check_name(db, name)
            date, msg_text = get_arguments(text_message, timer_type)
            attachment = ",".join([
                f"{attachment['type']}"
                f"{attachment[attachment['type']]['owner_id']}_"
                f"{attachment[attachment['type']]['id']}"
                for attachment in message.attachments
            ])
            if not msg_text and not attachment:
                raise E("⚠ Необходим текст сообщения ")
            db.timers.append(
                Timer(
                    name=name,
                    type=timer_type,
                    method=TimerMethod.SEND_MESSAGE,
                    peer_id=message.peer_id,
                    message_text=msg_text,
                    message_attachment=attachment,
                    run_date=date if timer_type == TimerType.DATE else None,
                    interval=date if timer_type == TimerType.INTERVAL else None,
                    cron=date if timer_type == TimerType.CRON else None
                )
            )
            await edit_message(
                message,
                "✅ Таймер успешно создан"
            )
        except E as ex:
            await edit_message(
                message,
                str(ex)
            )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> таймеры")
@logger_decorator
async def show_timers_wrapper(
        message: Message,
        **kwargs
):
    db = Database.get_current()
    text = "📃 Ваши таймеры:\n"
    for index, timer in enumerate(db.timers, 1):
        text += f"{index}. {timer.name}\n"
    await edit_message(
        message,
        text
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> !таймеры")
@logger_decorator
async def show_timers_from_scheduler_wrapper(
        message: Message,
        **kwargs
):
    text = "📃 Ваши таймеры (из планировщика):\n"
    for index, job in enumerate(const.scheduler.get_jobs(), 1):
        text += f"{index}. {job.name} {job.next_run_time.strftime('%Y.%m.%d %H:%M:%S')}\n"
    await edit_message(
        message,
        text
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +таймер <name> смс <text_message>")
@logger_decorator
async def add_date_timer_wrapper(
        message: Message,
        name: str,
        text_message: str,
        **kwargs
):
    await create_timer(message, name, text_message, TimerType.DATE)


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +цтаймер <name> смс <text_message>")
@logger_decorator
async def add_interval_timer_wrapper(
        message: Message,
        name: str,
        text_message: str,
        **kwargs
):
    await create_timer(message, name, text_message, TimerType.INTERVAL)


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +ктаймер <name> смс <text_message>")
@logger_decorator
async def add_cron_timer_wrapper(
        message: Message,
        name: str,
        text_message: str,
        **kwargs
):
    await create_timer(message, name, text_message, TimerType.CRON)


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -таймер <name>")
@logger_decorator
async def remove_timer_wrapper(
        message: Message,
        name: str,
        **kwargs
):
    with Database.get_current() as db:
        timer = None
        for tim in db.timers:
            if tim.name.lower() == name.lower():
                timer = tim
                break
        if not timer:
            await edit_message(
                message,
                "⚠ Таймер не найден"
            )
            return
        db.timers.remove(timer)
        await edit_message(
            message,
            f"⚠ Таймер <<{name}>> успешно удален"
        )



import re

from collections import namedtuple
from typing import Optional, NamedTuple

from vkbottle.user import Blueprint, Message
from vkbottle.rule import FromMe

from idm_lp import rules
from idm_lp.logger import logger_decorator
from idm_lp.database import Database
from idm_lp.utils import edit_message

user = Blueprint(
    name='bio_wars_blueprint'
)

RegexFindAllBase = namedtuple('RegexFindAll', ['regex', 'groups_map'])


class RegexFindAll(RegexFindAllBase):

    def match(self, text: str) -> Optional[NamedTuple]:
        re_result = re.findall(self.regex, text)
        if re_result:
            if isinstance(re_result[0], tuple):
                return namedtuple('RegexFindAllResult', self.groups_map)(*[str(res) for res in re_result[0]])
            else:
                return namedtuple('RegexFindAllResult', self.groups_map)(str(re_result[0]))
        return None


USER_ID_REGEX = RegexFindAll(
    re.compile(
        r'Организатор заражения: \[id(?P<user_id>\d+)',
        flags=re.MULTILINE & re.IGNORECASE
    ),
    ['user_id']
)


@user.on.message_handler(rules.ContainsRule(['Служба безопасности лаборатории']))
@logger_decorator
async def bio_reply_handler(message: Message):
    if message.from_id > 0:
        return

    db = Database.get_current()
    if not db.bio_reply:
        return

    if str(await message.api.user_id) not in message.text:
        return

    lab_user = USER_ID_REGEX.match(message.text)
    if lab_user:
        # noinspection PyUnresolvedReferences
        return f"Заразить @id{lab_user.user_id}"


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -заражение")
@logger_decorator
async def activate_bio_reply_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.bio_reply = False
    db.save()
    await edit_message(
        message,
        "✅ Заражение в ответ отключено"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +заражение")
@logger_decorator
async def deactivate_bio_reply_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.bio_reply = True
    db.save()
    await edit_message(
        message,
        "✅ Заражение в ответ включено"
    )
